@@ -10,8 +10,8 @@ import numpy as np
 
 from larray.core.axis import Axis
 from larray.core.array import LArray, larray_nan_equal, get_axes, ndtest, zeros, zeros_like, sequence
-from larray.util.misc import float_error_handler_factory, is_interactive_interpreter
-from larray.io.session import check_pattern, handler_classes, ext_default_engine
+from larray.util.misc import float_error_handler_factory, is_interactive_interpreter, renamed_to
+from larray.inout.session import check_pattern, handler_classes, ext_default_engine
 
 
 # XXX: inherit from OrderedDict or LArray?
@@ -97,7 +97,8 @@ class Session(object):
 
     def __getitem__(self, key):
         if isinstance(key, int):
-            return self._objects[self.keys()[key]]
+            keys = list(self.keys())
+            return self._objects[keys[key]]
         elif isinstance(key, LArray):
             assert np.issubdtype(key.dtype, np.bool_)
             assert key.ndim == 1
@@ -113,22 +114,19 @@ class Session(object):
     def get(self, key, default=None):
         """
         Returns the array object corresponding to the key.
-        If the key doesn't correspond to any array object,
-        a default one can be returned.
+        If the key doesn't correspond to any array object, a default one can be returned.
 
         Parameters
         ----------
         key : str
-            Name the array.
+            Name of the array.
         default : array, optional
-            Returned array if the key doesn't correspond
-            to any array of the current session.
+            Returned array if the key doesn't correspond to any array of the current session.
 
         Returns
         -------
         LArray
-            Array corresponding to the given key or
-            a default one if not found.
+            Array corresponding to the given key or a default one if not found.
 
         Examples
         --------
@@ -182,7 +180,7 @@ class Session(object):
 
     def load(self, fname, names=None, engine='auto', display=False, **kwargs):
         """
-        Loads array objects from a file.
+        Loads array objects from a file, or several .csv files.
 
         WARNING: never load a file using the pickle engine (.pkl or .pickle) from an untrusted source, as it can lead
         to arbitrary code execution.
@@ -190,24 +188,23 @@ class Session(object):
         Parameters
         ----------
         fname : str
-            Path to the file.
+            This can be either the path to a single file, a path to a directory containing .csv files or a pattern
+            representing several .csv files.
         names : list of str, optional
             List of arrays to load. If `fname` is None, list of paths to CSV files.
             Defaults to all valid objects present in the file/directory.
         engine : {'auto', 'pandas_csv', 'pandas_hdf', 'pandas_excel', 'xlwings_excel', 'pickle'}, optional
-            Load using `engine`. Defaults to 'auto' (use default engine for
-            the format guessed from the file extension).
+            Load using `engine`. Defaults to 'auto' (use default engine for the format guessed from the file extension).
         display : bool, optional
-            whether or not to display which file is being worked on. Defaults
-            to False.
+            Whether or not to display which file is being worked on. Defaults to False.
 
         Examples
         --------
         In one module
 
-        >>> arr1, arr2, arr3 = ndtest((2, 2)), ndtest(4), ndtest((3, 2))  # doctest: +SKIP
-        >>> s = Session([('arr1', arr1), ('arr2', arr2), ('arr3', arr3)]) # doctest: +SKIP
-        >>> s.save('input.h5')  # doctest: +SKIP
+        >>> arr1, arr2, arr3 = ndtest((2, 2)), ndtest(4), ndtest((3, 2))   # doctest: +SKIP
+        >>> s = Session([('arr1', arr1), ('arr2', arr2), ('arr3', arr3)])  # doctest: +SKIP
+        >>> s.save('input.h5')                                             # doctest: +SKIP
 
         In another module
 
@@ -216,6 +213,15 @@ class Session(object):
         >>> arr1, arr2, arr3 = s['arr1', 'arr2', 'arr3']  # doctest: +SKIP
         >>> # only if you know the order of arrays stored in session
         >>> arr1, arr2, arr3 = s.values()                 # doctest: +SKIP
+
+        Using .csv files (assuming the same session as above)
+
+        >>> s.save('data')                                # doctest: +SKIP
+        >>> s = Session()                                 # doctest: +SKIP
+        >>> # load all .csv files starting with "output" in the data directory
+        >>> s.load('data')                                # doctest: +SKIP
+        >>> # or equivalently in this case
+        >>> s.load('data/arr*.csv')                       # doctest: +SKIP
         """
         if display:
             print("opening", fname)
@@ -246,11 +252,10 @@ class Session(object):
             List of names of objects to dump. If `fname` is None, list of paths to CSV files.
             Defaults to all objects present in the Session.
         engine : {'auto', 'pandas_csv', 'pandas_hdf', 'pandas_excel', 'xlwings_excel', 'pickle'}, optional
-            Dump using `engine`. Defaults to 'auto' (use default engine for
-            the format guessed from the file extension).
+            Dump using `engine`. Defaults to 'auto' (use default engine for the format guessed from the file extension).
         overwrite: bool, optional
-            Whether or not to overwrite an existing file, if any. Ignored for CSV files. 
-            If False, file is updated. Defaults to True.
+            Whether or not to overwrite an existing file, if any. Ignored for CSV files. If False, file is updated.
+            Defaults to True.
         display : bool, optional
             Whether or not to display which file is being worked on. Defaults to False.
 
@@ -266,9 +271,9 @@ class Session(object):
         Save only some arrays
 
         >>> s.save('output.h5', ['arr1', 'arr3'])  # doctest: +SKIP
-        
+
         Update file
-        
+
         >>> arr1, arr4 = ndtest((3, 3)), ndtest((2, 3))     # doctest: +SKIP
         >>> s2 = Session([('arr1', arr1), ('arr4', arr4)])  # doctest: +SKIP
         >>> # replace arr1 and add arr4 in file output.h5
@@ -278,17 +283,15 @@ class Session(object):
             _, ext = os.path.splitext(fname)
             ext = ext.strip('.') if '.' in ext else 'csv'
             engine = ext_default_engine[ext]
-        if overwrite and engine != ext_default_engine['csv'] and os.path.isfile(fname):
-            os.remove(fname)
         handler_cls = handler_classes[engine]
-        handler = handler_cls(fname)
+        handler = handler_cls(fname, overwrite)
         items = self.filter(kind=LArray).items()
         if names is not None:
             names_set = set(names)
             items = [(k, v) for k, v in items if k in names_set]
         handler.dump_arrays(items, display=display, **kwargs)
 
-    def to_globals(self, names=None, depth=0, warn=True):
+    def to_globals(self, names=None, depth=0, warn=True, inplace=False):
         """
         Create global variables out of objects in the session.
 
@@ -301,10 +304,14 @@ class Session(object):
             to_globals, etc. Defaults to 0.
         warn : bool, optional
             Whether or not to warn the user that this method should only be used in an interactive console (see below).
+            Defaults to True.
+        inplace : bool, optional
+            If True, to_globals will assume all arrays already exist and have the same axes and will replace their
+            content instead of creating new variables. Non array variables in the session will be ignored.
+            Defaults to False.
 
         Notes
         -----
-
         This method should usually only be used in an interactive console and not in a script. Code editors are
         confused by this kind of manipulation and will likely consider as invalid the code using variables created in
         this way. Additionally, when using this method auto-completion, "show definition", "go to declaration" and
@@ -313,7 +320,6 @@ class Session(object):
 
         Examples
         --------
-
         >>> s = Session(arr1=ndtest(3), arr2=ndtest((2, 2)))
         >>> s.to_globals()
         >>> arr1
@@ -334,8 +340,21 @@ class Session(object):
         if names is not None:
             names_set = set(names)
             items = [(k, v) for k, v in items if k in names_set]
-        for k, v in items:
-            d[k] = v
+        if inplace:
+            for k, v in items:
+                if k not in d:
+                    raise ValueError("'{}' not found in current namespace. Session.to_globals(inplace=True) requires "
+                                     "all arrays to already exist.".format(k))
+                if not isinstance(v, LArray):
+                    continue
+                if not d[k].axes == v.axes:
+                    raise ValueError("Session.to_globals(inplace=True) requires the existing (destination) arrays "
+                                     "to have the same axes than those stored in the session and this is not the case "
+                                     "for '{}'.\nexisting: {}\nsession: {}".format(k, d[k].info, v.info))
+                d[k][:] = v
+        else:
+            for k, v in items:
+                d[k] = v
 
     def to_pickle(self, fname, names=None, overwrite=True, display=False, **kwargs):
         """
@@ -349,10 +368,9 @@ class Session(object):
         fname : str
             Path for the dump.
         names : list of str or None, optional
-            List of names of objects to dump. Defaults to all objects
-            present in the Session.
+            List of names of objects to dump. Defaults to all objects present in the Session.
         overwrite: bool, optional
-            Whether or not to overwrite an existing file, if any. 
+            Whether or not to overwrite an existing file, if any.
             If False, file is updated. Defaults to True.
         display : bool, optional
             Whether or not to display which file is being worked on. Defaults to False.
@@ -372,9 +390,7 @@ class Session(object):
         """
         self.save(fname, names, ext_default_engine['pkl'], overwrite, display, **kwargs)
 
-    def dump(self, fname, names=None, engine='auto', display=False, **kwargs):
-        warnings.warn("Method dump is deprecated. Use method save instead.", DeprecationWarning, stacklevel=2)
-        self.save(fname, names, engine, display, **kwargs)
+    dump = renamed_to(save, 'dump')
 
     def to_hdf(self, fname, names=None, overwrite=True, display=False, **kwargs):
         """
@@ -385,10 +401,9 @@ class Session(object):
         fname : str
             Path for the dump.
         names : list of str or None, optional
-            List of names of objects to dump. Defaults to all objects
-            present in the Session.
+            List of names of objects to dump. Defaults to all objects present in the Session.
         overwrite: bool, optional
-            Whether or not to overwrite an existing file, if any. 
+            Whether or not to overwrite an existing file, if any.
             If False, file is updated. Defaults to True.
         display : bool, optional
             Whether or not to display which file is being worked on. Defaults to False.
@@ -408,9 +423,7 @@ class Session(object):
         """
         self.save(fname, names, ext_default_engine['hdf'], overwrite, display, **kwargs)
 
-    def dump_hdf(self, fname, names=None, *args, **kwargs):
-        warnings.warn("Method dump_hdf is deprecated. Use method to_hdf instead.", DeprecationWarning, stacklevel=2)
-        self.to_hdf(fname, names, *args, **kwargs)
+    dump_hdf = renamed_to(to_hdf, 'dump_hdf')
 
     def to_excel(self, fname, names=None, overwrite=True, display=False, **kwargs):
         """
@@ -421,11 +434,9 @@ class Session(object):
         fname : str
             Path for the dump.
         names : list of str or None, optional
-            List of names of objects to dump. Defaults to all objects
-            present in the Session.
+            List of names of objects to dump. Defaults to all objects present in the Session.
         overwrite: bool, optional
-            Whether or not to overwrite an existing file, if any. 
-            If False, file is updated. Defaults to True.
+            Whether or not to overwrite an existing file, if any. If False, file is updated. Defaults to True.
         display : bool, optional
             Whether or not to display which file is being worked on. Defaults to False.
 
@@ -444,9 +455,7 @@ class Session(object):
         """
         self.save(fname, names, ext_default_engine['xlsx'], overwrite, display, **kwargs)
 
-    def dump_excel(self, fname, names=None, *args, **kwargs):
-        warnings.warn("Method dump_excel is deprecated. Use method to_excel instead.", DeprecationWarning, stacklevel=2)
-        self.to_excel(fname, names, *args, **kwargs)
+    dump_excel = renamed_to(to_excel, 'dump_excel')
 
     def to_csv(self, fname, names=None, display=False, **kwargs):
         """
@@ -457,8 +466,7 @@ class Session(object):
         fname : str
             Path for the directory that will contain CSV files.
         names : list of str or None, optional
-            List of names of objects to dump. Defaults to all objects
-            present in the Session.
+            List of names of objects to dump. Defaults to all objects present in the Session.
         display : bool, optional
             Whether or not to display which file is being worked on. Defaults to False.
 
@@ -477,9 +485,7 @@ class Session(object):
         """
         self.save(fname, names, ext_default_engine['csv'], display=display, **kwargs)
 
-    def dump_csv(self, fname, names=None, *args, **kwargs):
-        warnings.warn("Method dump_csv is deprecated. Use method to_csv instead.", DeprecationWarning, stacklevel=2)
-        self.to_csv(fname, names, *args, **kwargs)
+    dump_csv = renamed_to(to_csv, 'dump_csv')
 
     def filter(self, pattern=None, kind=None):
         """
@@ -614,17 +620,22 @@ class Session(object):
         Examples
         --------
         >>> arr1, arr2, arr3 = ndtest((2, 2)), ndtest(4), ndtest((3, 2))
+        >>> # make the test pass on both Windows and Linux
+        >>> arr1, arr2, arr3 = arr1.astype(np.int64), arr2.astype(np.int64), arr3.astype(np.int64)
         >>> s = Session([('arr2', arr2), ('arr1', arr1), ('arr3', arr3)])
         >>> for k, v in s.items():
         ...     print("{}: {}".format(k, v.info))
         arr2: 4
          a [4]: 'a0' 'a1' 'a2' 'a3'
+        dtype: int64
         arr1: 2 x 2
          a [2]: 'a0' 'a1'
          b [2]: 'b0' 'b1'
+        dtype: int64
         arr3: 3 x 2
          a [3]: 'a0' 'a1' 'a2'
          b [2]: 'b0' 'b1'
+        dtype: int64
         """
         return self._objects.items()
 
@@ -634,21 +645,24 @@ class Session(object):
     def __len__(self):
         return len(self._objects)
 
-    # binary operations are dispatched element-wise to all arrays
-    # (we consider Session as an array-like)
+    # binary operations are dispatched element-wise to all arrays (we consider Session as an array-like)
     def _binop(opname):
         opfullname = '__%s__' % opname
 
         def opmethod(self, other):
             self_keys = set(self.keys())
-            all_keys = list(self.keys()) + [n for n in other.keys() if
-                                            n not in self_keys]
+            all_keys = list(self.keys()) + [n for n in other.keys() if n not in self_keys]
             with np.errstate(call=_session_float_error_handler):
                 res = []
                 for name in all_keys:
                     self_array = self.get(name, np.nan)
                     other_array = other.get(name, np.nan)
-                    res.append((name, getattr(self_array, opfullname)(other_array)))
+                    try:
+                        res_array = getattr(self_array, opfullname)(other_array)
+                    # TypeError for str arrays, ValueError for incompatible axes, ...
+                    except Exception:
+                        res_array = np.nan
+                    res.append((name, res_array))
             return Session(res)
         opmethod.__name__ = opfullname
         return opmethod
@@ -658,22 +672,97 @@ class Session(object):
     __mul__ = _binop('mul')
     __truediv__ = _binop('truediv')
 
-    # XXX: use _binop (ie elementwise comparison instead of aggregating
-    #      directly?)
+    # element-wise method factory
+    # unary operations are (also) dispatched element-wise to all arrays
+    def _unaryop(opname):
+        opfullname = '__%s__' % opname
+
+        def opmethod(self):
+            with np.errstate(call=_session_float_error_handler):
+                res = []
+                for k, v in self.items():
+                    try:
+                        res_array = getattr(v, opfullname)()
+                    except Exception:
+                        res_array = np.nan
+                    res.append((k, res_array))
+            return Session(res)
+        opmethod.__name__ = opfullname
+        return opmethod
+
+    __neg__ = _unaryop('neg')
+    __pos__ = _unaryop('pos')
+    __abs__ = _unaryop('abs')
+    __invert__ = _unaryop('invert')
+
+    # XXX: use _binop (ie elementwise comparison instead of aggregating directly?)
     def __eq__(self, other):
         self_keys = set(self.keys())
-        all_keys = list(self.keys()) + [n for n in other.keys()
-                                        if n not in self_keys]
+        all_keys = list(self.keys()) + [n for n in other.keys() if n not in self_keys]
         res = [larray_nan_equal(self.get(key), other.get(key)) for key in all_keys]
         return LArray(res, [Axis(all_keys, 'name')])
 
     def __ne__(self, other):
         return ~(self == other)
 
+    def transpose(self, *args):
+        """Reorder axes of arrays in session, ignoring missing axes for each array.
+
+        Parameters
+        ----------
+        *args
+            Accepts either a tuple of axes specs or axes specs as `*args`. Omitted axes keep their order.
+            Use ... to avoid specifying intermediate axes. Axes missing in an array are ignored.
+
+        Returns
+        -------
+        Session
+            Session with each array with reordered axes where appropriate.
+
+        See Also
+        --------
+        LArray.transpose
+
+        Examples
+        --------
+        Let us create a test session and a small helper function to display sessions as a short summary.
+
+        >>> arr1 = ndtest((2, 2, 2))
+        >>> arr2 = ndtest((2, 2))
+        >>> sess = Session([('arr1', arr1), ('arr2', arr2)])
+        >>> def print_summary(s):
+        ...     print(s.summary("{name} -> {axes_names}"))
+        >>> print_summary(sess)
+        arr1 -> a, b, c
+        arr2 -> a, b
+
+        Put 'b' axis in front of all arrays
+
+        >>> print_summary(sess.transpose('b'))
+        arr1 -> b, a, c
+        arr2 -> b, a
+
+        Axes missing on an array are ignored ('c' for arr2 in this case)
+
+        >>> print_summary(sess.transpose('c', 'b'))
+        arr1 -> c, b, a
+        arr2 -> b, a
+
+        Use ... to move axes to the end
+
+        >>> print_summary(sess.transpose(..., 'a'))   # doctest: +SKIP
+        arr1 -> b, c, a
+        arr2 -> b, a
+        """
+        def lenient_transpose(v, axes):
+            # filter out axes not in arr.axes
+            return v.transpose([a for a in axes if a in v.axes or a is Ellipsis])
+        return self.apply(lenient_transpose, args)
+
     def compact(self, display=False):
         """
-        Detects and removes "useless" axes (ie axes for which values are
-        constant over the whole axis) for all array objects in session
+        Detects and removes "useless" axes (ie axes for which values are constant over the whole axis) for all array
+        objects in session
 
         Parameters
         ----------
@@ -707,6 +796,67 @@ class Session(object):
             new_items.append((k, compacted))
         return Session(new_items)
 
+    def apply(self, func, *args, **kwargs):
+        """
+        Apply function `func` on elements of the session and return a new session.
+
+        Parameters
+        ----------
+        func : function
+            Function to apply to each element of the session. It should take a single `element` argument and return
+            a single value.
+        *args : any
+            Any extra arguments are passed to the function
+        kind : type or tuple of types, optional
+            Type(s) of elements `func` will be applied to. Other elements will be left intact. Use ´kind=object´ to
+            apply to all kinds of objects. Defaults to LArray.
+        **kwargs : any
+            Any extra keyword arguments are passed to the function
+
+        Returns
+        -------
+        Session
+            A new session containing all processed elements
+
+        Examples
+        --------
+        >>> arr1 = ndtest(2)
+        >>> arr1
+        a  a0  a1
+            0   1
+        >>> arr2 = ndtest(3)
+        >>> arr2
+        a  a0  a1  a2
+            0   1   2
+        >>> sess1 = Session([('arr1', arr1), ('arr2', arr2)])
+        >>> sess1
+        Session(arr1, arr2)
+        >>> def increment(array):
+        ...     return array + 1
+        >>> sess2 = sess1.apply(increment)
+        >>> sess2.arr1
+        a  a0  a1
+            1   2
+        >>> sess2.arr2
+        a  a0  a1  a2
+            1   2   3
+
+        You may also pass extra arguments or keyword arguments to the function
+
+        >>> def change(array, increment=1, multiplier=1):
+        ...     return (array + increment) * multiplier
+        >>> sess2 = sess1.apply(change, 2, 2)
+        >>> sess2 = sess1.apply(change, 2, multiplier=2)
+        >>> sess2.arr1
+        a  a0  a1
+            4   6
+        >>> sess2.arr2
+        a  a0  a1  a2
+            4   6   8
+        """
+        kind = kwargs.pop('kind', LArray)
+        return Session([(k, func(v, *args, **kwargs) if isinstance(v, kind) else v) for k, v in self.items()])
+
     def summary(self, template=None):
         """
         Returns a summary of the content of the session.
@@ -750,13 +900,12 @@ class Session(object):
 
 def local_arrays(depth=0):
     """
-    Returns a session containing all local arrays (sorted in alphabetical order).
+    Returns a session containing all local arrays sorted in alphabetical order.
 
     Parameters
     ----------
     depth: int
-        depth of call frame to inspect. 0 is where local_arrays was called,
-        1 the caller of local_arrays, etc.
+        depth of call frame to inspect. 0 is where `local_arrays` was called, 1 the caller of `local_arrays`, etc.
 
     Returns
     -------
@@ -767,4 +916,50 @@ def local_arrays(depth=0):
     return Session((k, d[k]) for k in sorted(d.keys()) if isinstance(d[k], LArray))
 
 
+def global_arrays(depth=0):
+    """
+    Returns a session containing all global arrays sorted in alphabetical order.
+
+    Parameters
+    ----------
+    depth: int
+        depth of call frame to inspect. 0 is where `global_arrays` was called, 1 the caller of `global_arrays`, etc.
+
+    Returns
+    -------
+    Session
+    """
+    # noinspection PyProtectedMember
+    d = sys._getframe(depth + 1).f_globals
+    return Session((k, d[k]) for k in sorted(d.keys()) if isinstance(d[k], LArray))
+
+
+def arrays(depth=0):
+    """
+    Returns a session containing all available arrays (whether they are defined in local or global variables) sorted in
+    alphabetical order. Local arrays take precedence over global ones (if a name corresponds to both a local
+    and a global variable, the local array will be returned).
+
+    Parameters
+    ----------
+    depth: int
+        depth of call frame to inspect. 0 is where `arrays` was called, 1 the caller of `arrays`, etc.
+
+    Returns
+    -------
+    Session
+    """
+    # noinspection PyProtectedMember
+    caller_frame = sys._getframe(depth + 1)
+    global_vars = caller_frame.f_globals
+    local_vars = caller_frame.f_locals
+
+    # We must first get all variables *then* filter by type, otherwise we could return a global array which is not
+    # currently available because it is shadowed by a local non-array variable.
+    all_keys = sorted(set(global_vars.keys()) | set(local_vars.keys()))
+    combined_vars = [(k, local_vars[k] if k in local_vars else global_vars[k])
+                     for k in all_keys]
+    return Session((k, v) for k, v in combined_vars if isinstance(v, LArray))
+
+
 _session_float_error_handler = float_error_handler_factory(4)
