@@ -82,8 +82,7 @@ def from_series(s, sort_rows=False):
     s : Pandas Series
         Input Pandas Series.
     sort_rows : bool, optional
-        Whether or not to sort the rows alphabetically (sorting is more efficient than not sorting).
-        Defaults to False.
+        Whether or not to sort the rows alphabetically. Defaults to False.
 
     Returns
     -------
@@ -92,9 +91,9 @@ def from_series(s, sort_rows=False):
     name = s.name if s.name is not None else s.index.name
     if name is not None:
         name = str(name)
-    labels = sorted(s.index.values) if sort_rows else list(s.index.values)
-    axis = Axis(labels, name)
-    return LArray(s.values, axis)
+    if sort_rows:
+        s = s.sort_index()
+    return LArray(s.values, Axis(s.index.values, name))
 
 
 def from_frame(df, sort_rows=False, sort_columns=False, parse_header=True, unfold_last_axis_name=False, **kwargs):
@@ -234,9 +233,13 @@ def df_aslarray(df, sort_rows=False, sort_columns=False, raw=False, parse_header
     # handle 1D
     if len(df) == 1 and (pd.isnull(df.index.values[0]) or
                              (isinstance(df.index.values[0], basestring) and df.index.values[0].strip() == '')):
+        if parse_header:
+            df.columns = pd.Index([parse(cell) for cell in df.columns.values], name=df.columns.name)
         series = df.iloc[0]
         series.name = df.index.name
-        return from_series(series, sort_rows=sort_rows)
+        if sort_rows:
+            raise ValueError('sort_rows=True is not valid for 1D arrays. Please use sort_columns instead.')
+        return from_series(series, sort_rows=sort_columns)
     else:
         axes_names = [decode(name, 'utf8') for name in df.index.names]
         unfold_last_axis_name = isinstance(axes_names[-1], basestring) and '\\' in axes_names[-1]
