@@ -6,7 +6,7 @@ import pytest
 import numpy as np
 
 from larray.tests.common import abspath, assert_array_equal, assert_array_nan_equal
-from larray import Axis, AxisCollection, LGroup, PGroup
+from larray import Axis, AxisCollection, LGroup, IGroup
 
 
 class TestAxis(TestCase):
@@ -21,6 +21,10 @@ class TestAxis(TestCase):
         sex_list = ['M', 'F']
         sex_array = np.array(sex_list)
 
+        # wildcard axis
+        axis = Axis(10, 'axis')
+        assert len(axis) == 10
+        assert list(axis.labels) == list(range(10))
         # tuple of strings
         assert_array_equal(Axis(sex_tuple, 'sex').labels, sex_array)
         # list of strings
@@ -39,6 +43,11 @@ class TestAxis(TestCase):
         group_axis = Axis(group)
         assert_array_equal(group_axis.labels, np.arange(11))
         assert_array_equal(group_axis.name, 'age')
+        # another axis as labels argument
+        other = Axis('other=0..10')
+        axis = Axis(other, 'age')
+        assert_array_equal(axis.labels, other.labels)
+        assert_array_equal(axis.name, 'age')
 
     def test_equals(self):
         self.assertTrue(Axis('sex=M,F').equals(Axis('sex=M,F')))
@@ -88,6 +97,18 @@ class TestAxis(TestCase):
         self.assertEqual(group.key, slice(None))
         self.assertIs(group.axis, age)
 
+        # an axis
+        age2 = Axis('age=0..5')
+        group = age[age2]
+        assert list(group.key) == list(age2.labels)
+
+    def test_translate(self):
+        # an axis with labels having the object dtype
+        a = Axis(np.array(["a0", "a1"], dtype=object), 'a')
+
+        self.assertEqual(a.translate('a1'), 1)
+        self.assertEqual(a.translate('a1 >> A1'), 1)
+
     def test_getitem_lgroup_keys(self):
         def group_equal(g1, g2):
             return (g1.key == g2.key and g1.name == g2.name and
@@ -152,7 +173,7 @@ class TestAxis(TestCase):
         self.assertEqual(g.key, ['a1', 'a2'])
         self.assertIs(g.axis, alt_a)
 
-        # b) key is a single PGroup
+        # b) key is a single IGroup
         # -------------------------
 
         # b.1) containing a scalar
@@ -211,7 +232,7 @@ class TestAxis(TestCase):
         self.assertEqual(g.key, slice('a1', 'a2'))
         self.assertIs(g.axis, alt_a)
 
-        # c.2) with PGroup bounds
+        # c.2) with IGroup bounds
         pg_a1 = a.i[1]
         pg_a2 = a.i[2]
         # use it on the same axis
@@ -241,7 +262,7 @@ class TestAxis(TestCase):
         self.assertEqual(g.key, ['a1', 'a2'])
         self.assertIs(g.axis, alt_a)
 
-        # d.2) with PGroup
+        # d.2) with IGroup
         key = [a.i[1], a.i[2]]
         # use it on the same axis
         g = a[key]
@@ -278,7 +299,7 @@ class TestAxis(TestCase):
         self.assertIs(g[0].axis, alt_a)
         self.assertIs(g[1].axis, alt_a)
 
-        # e.2) with PGroup
+        # e.2) with IGroup
         key = (a.i[1, 2], a.i[2, 1])
         # use it on the same axis => change to LGroup
         g = a[key]
@@ -315,7 +336,7 @@ class TestAxis(TestCase):
         self.assertEqual(g.key, ['a1', 'a2'])
         self.assertIs(g.axis, alt_a)
 
-        # f.2) with PGroup
+        # f.2) with IGroup
         key = (a.i[1], a.i[2])
         # use it on the same axis
         g = a[key]
@@ -352,7 +373,7 @@ class TestAxis(TestCase):
         self.assertIs(g[0].axis, alt_a)
         self.assertIs(g[1].axis, alt_a)
 
-        # g.2) with PGroup
+        # g.2) with IGroup
         key = (a.i[1, 2], a.i[2, 1])
         # use it on the same axis
         g = a[key]
@@ -379,15 +400,15 @@ class TestAxis(TestCase):
         subset_axis = Axis(code_group, 'code_subset')
         assert_array_equal(subset_axis.labels, ['C01', 'C02'])
 
-    def test_match(self):
+    def test_matching(self):
         sutcode = Axis(['A23', 'A2301', 'A25', 'A2501'], 'sutcode')
-        self.assertEqual(sutcode.matches('^...$'), LGroup(['A23', 'A25']))
-        self.assertEqual(sutcode.startswith('A23'), LGroup(['A23', 'A2301']))
-        self.assertEqual(sutcode.endswith('01'), LGroup(['A2301', 'A2501']))
+        self.assertEqual(sutcode.matching('^...$'), LGroup(['A23', 'A25']))
+        self.assertEqual(sutcode.startingwith('A23'), LGroup(['A23', 'A2301']))
+        self.assertEqual(sutcode.endingwith('01'), LGroup(['A2301', 'A2501']))
 
     def test_iter(self):
         sex = Axis('sex=M,F')
-        self.assertEqual(list(sex), [PGroup(0, axis=sex), PGroup(1, axis=sex)])
+        self.assertEqual(list(sex), [IGroup(0, axis=sex), IGroup(1, axis=sex)])
 
     def test_positional(self):
         age = Axis('age=0..115')
@@ -510,6 +531,13 @@ class TestAxisCollection(TestCase):
         assert_array_equal(col2.lipro.labels, ['P01', 'P02', 'P03'])
         assert_array_equal(col2.sex.labels, ['M', 'F'])
 
+    def test_init_from_string(self):
+        col = AxisCollection('age=10;sex=M,F;year=2000..2017')
+        assert col.names == ['age', 'sex', 'year']
+        assert list(col.age.labels) == [10]
+        assert list(col.sex.labels) == ['M', 'F']
+        assert list(col.year.labels) == [y for y in range(2000, 2018)]
+
     def test_eq(self):
         col = self.collection
         self.assertEqual(col, col)
@@ -676,7 +704,77 @@ class TestAxisCollection(TestCase):
         self.assertEqual(newcol.names, ['lipro', 'letters', 'age'])
         self.assertEqual(newcol.shape, (4, 3, 8))
 
-    # TODO: add contains_test (using both axis name and axis objects)
+    def test_contains(self):
+        col = self.collection
+        self.assertTrue('lipro' in col)
+        self.assertFalse('nonexisting' in col)
+
+        self.assertTrue(0 in col)
+        self.assertTrue(1 in col)
+        self.assertTrue(2 in col)
+        self.assertTrue(-1 in col)
+        self.assertTrue(-2 in col)
+        self.assertTrue(-3 in col)
+        self.assertFalse(3 in col)
+
+        # objects actually in col
+        self.assertTrue(self.lipro in col)
+        self.assertTrue(self.sex in col)
+        self.assertTrue(self.age in col)
+        # other axis with the same name
+        self.assertTrue(self.sex2 in col)
+        self.assertFalse(self.geo in col)
+        self.assertFalse(self.value in col)
+
+        # test anonymous axes
+        anon = Axis([0, 1])
+        col.append(anon)
+        self.assertTrue(anon in col)
+        # different object, same values
+        anon2 = anon.copy()
+        self.assertTrue(anon2 in col)
+        # different values
+        anon3 = Axis([0, 2])
+        self.assertFalse(anon3 in col)
+
+    def test_index(self):
+        col = self.collection
+        self.assertEqual(col.index('lipro'), 0)
+        with self.assertRaises(ValueError):
+            col.index('nonexisting')
+        self.assertEqual(col.index(0), 0)
+        self.assertEqual(col.index(1), 1)
+        self.assertEqual(col.index(2), 2)
+        self.assertEqual(col.index(-1), -1)
+        self.assertEqual(col.index(-2), -2)
+        self.assertEqual(col.index(-3), -3)
+        with self.assertRaises(ValueError):
+            col.index(3)
+
+        # objects actually in col
+        self.assertEqual(col.index(self.lipro), 0)
+        self.assertEqual(col.index(self.sex), 1)
+        self.assertEqual(col.index(self.age), 2)
+        # other axis with the same name
+        self.assertEqual(col.index(self.sex2), 1)
+        # non existing
+        with self.assertRaises(ValueError):
+            col.index(self.geo)
+        with self.assertRaises(ValueError):
+            col.index(self.value)
+
+        # test anonymous axes
+        anon = Axis([0, 1])
+        col.append(anon)
+        self.assertEqual(col.index(anon), 3)
+        # different object, same values
+        anon2 = anon.copy()
+        self.assertEqual(col.index(anon2), 3)
+        # different values
+        anon3 = Axis([0, 2])
+        with self.assertRaises(ValueError):
+            col.index(anon3)
+
     def test_get(self):
         col = self.collection
         self.assert_axis_eq(col.get('lipro'), self.lipro)
